/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.access;

import com.carlosbcruz.access.model.ComplexParameter;
import com.carlosbcruz.access.model.Function;
import com.carlosbcruz.access.model.SimpleParameter;
import com.sap.mw.jco.IFunctionTemplate;
import com.sap.mw.jco.IMetaData;
import com.sap.mw.jco.IRepository;
import com.sap.mw.jco.JCO;

/**
 * Manage the access to SAP functions
 * 
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class Access {

    private static String POOL_MANAGER = "PoolManager";

    private static String ENGLISH_POOL = "EnglishPool";

    private static String FRENCH_POOL = "FrenchPool";

    private static String JAPANESE_POOL = "JapanesePool";

    private static String REPOSITORY = "Repository";

    /**
     * Create the connection pools
     */
    public static void createConnectionPools(String sapMode, String sapClient,
        String groupName, String hostName, String systemName, String password) {

        JCO.Pool pool = JCO.getClientPoolManager().getPool(POOL_MANAGER);

        if (pool == null) {

            JCO.addClientPool(ENGLISH_POOL, Integer.parseInt(sapMode),
                sapClient, groupName, password, "en", hostName, systemName,
                groupName);
            JCO.addClientPool(FRENCH_POOL, Integer.parseInt(sapMode),
                sapClient, groupName, password, "fr", hostName, systemName,
                groupName);
            JCO.addClientPool(JAPANESE_POOL, Integer.parseInt(sapMode),
                sapClient, groupName, password, "ja", hostName, systemName,
                groupName);

        }

    }

    /**
     * Document a specific function 
     * @param repository the repository
     * @param parms the parameters
     * @param i the level
     */
    private static Function documentComplexParameter(IRepository repository,
        IMetaData parms, int i, Function function) {

        IMetaData arg = parms.getMetaData(i);

        ComplexParameter complexParameter = new ComplexParameter();
        String typeName = arg.getName();
        complexParameter.setName(typeName);

        if (parms.getType(i) == JCO.TYPE_TABLE) {
            function.addComment("Table: " + typeName);
            complexParameter.setType(ComplexParameter.TABLE_TYPE);
        } else
            if (parms.getType(i) == JCO.TYPE_STRUCTURE) {
                function.addComment("Structure: " + typeName);
                complexParameter.setType(ComplexParameter.STRUCTURE_TYPE);
            }

        IMetaData typeMetaData;
        if (parms.getType(i) == JCO.TYPE_STRUCTURE)
            typeMetaData = repository.getStructureDefinition(typeName);
        else
            typeMetaData = repository.getTableDefinition(typeName);

        for (int j = 0; j < typeMetaData.getFieldCount(); j++) {

            String memberTypeName;
            
            if (typeMetaData.getType(j) == JCO.TYPE_TABLE
                || typeMetaData.getType(j) == JCO.TYPE_STRUCTURE)
                memberTypeName = typeMetaData.getMetaData(j).getName();
            else
                memberTypeName = typeMetaData.getTypeAsString(j);

            function.addComment("      " + typeMetaData.getName(j) + " "
                + memberTypeName + "(" + typeMetaData.getLength(j) + ") - "
                + typeMetaData.getDescription(j));

            SimpleParameter simpleParameter = new SimpleParameter();
            simpleParameter.setName(typeMetaData.getName(j));
            simpleParameter.setLength(typeMetaData.getLength(j));
            simpleParameter.setDescription(typeMetaData.getDescription(j));
            simpleParameter.setType(memberTypeName);

            complexParameter.addParameters(simpleParameter);

        }

        function.addComplexParameter(complexParameter);
        return function;

    }

    /**
     * Document a function parameter
     * @param parms the parameter
     */
    private static Function documentFunctionParameters(IRepository repository,
        IMetaData parms, Function function) {

        // Invalid parameters
        if (parms == null) {
            function.addComment("No export parameters.");
            return function;
        }

        // No parameters
        if (parms.getFieldCount() == 0) {
            function.addComment("No parameters");
            return function;
        }

        // Show the parameters
        for (int i = 0; i < parms.getFieldCount(); i++) {

            if (parms.getType(i) == JCO.TYPE_TABLE
                || parms.getType(i) == JCO.TYPE_STRUCTURE) {

                function = documentComplexParameter(repository, parms, i,
                    function);

            } else {
                
                function.addComment("  " + parms.getName(i) + " "
                    + parms.getTypeAsString(i) + "(" + parms.getLength(i)
                    + ") - " + parms.getDescription(i));

                SimpleParameter simpleParameter = new SimpleParameter();
                simpleParameter.setName(parms.getName(i));
                simpleParameter.setLength(parms.getLength(i));
                simpleParameter.setDescription(parms.getDescription(i));
                simpleParameter.setType(parms.getTypeAsString(i));

                function.addSimpleParameter(simpleParameter);

            }
        }

        return function;

    }

    /**
     * Access a specific function and show its attributes and parameters
     * 
     * @param functionName the requested function
     */
    public static Function createFunctionModelFromSAP(Function function) {

        IRepository repository = JCO.createRepository(REPOSITORY, ENGLISH_POOL);

        IFunctionTemplate functionTemplate = repository
                .getFunctionTemplate(function.getName());

        function.addComment("Function: " + functionTemplate.getName());
        function.addComment("Import Parameters");

        IMetaData parms = functionTemplate.getImportParameterList();
        function = documentFunctionParameters(repository, parms, function);

        parms = functionTemplate.getExportParameterList();
        function.addComment("Export Parameters");
        function = documentFunctionParameters(repository, parms, function);

        function.addComment("Table Parameters");
        parms = functionTemplate.getTableParameterList();
        function = documentFunctionParameters(repository, parms, function);

        return function;
    }
}
