/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.sapproxy.generator;

import java.util.ArrayList;

import com.carlosbcruz.sapproxy.generator.model.ComplexParameter;
import com.carlosbcruz.sapproxy.generator.model.Function;
import com.carlosbcruz.sapproxy.generator.model.SimpleParameter;
import com.carlosbcruz.util.Err;
import com.carlosbcruz.util.ExceptionSupport;
import com.carlosbcruz.util.Out;
import com.sap.conn.jco.JCoDestination;
import com.sap.conn.jco.JCoDestinationManager;
import com.sap.conn.jco.JCoException;
import com.sap.conn.jco.JCoFunctionTemplate;
import com.sap.conn.jco.JCoListMetaData;
import com.sap.conn.jco.JCoMetaData;
import com.sap.conn.jco.JCoRecordMetaData;

/**
 * Manage the documentation of SAP functions
 *  
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class FunctionDocumentation {

	/**
	 * Access a specific function and show its attributes and parameters
	 * 
	 * @param functionName
	 *            the requested function
	 */
	public static Function createFunctionModelFromSAP(Function function) {

		try {
			JCoDestination destination = JCoDestinationManager
					.getDestination("SystemDestinationDataProvider");

			JCoFunctionTemplate functionTemplate = destination.getRepository()
					.getFunctionTemplate(function.getName());
			if (functionTemplate == null) {
				String message = function.getName() + " not found in SAP.";
				Err.print(message);
				ExceptionSupport.handleException(message);
				throw new RuntimeException(message);
			}

			function.addComment("Function: " + functionTemplate.getName());
			Out.println("Function: " + functionTemplate.getName());

			ArrayList<SimpleParameter> parameterList = new ArrayList<SimpleParameter>();

			JCoListMetaData params = functionTemplate.getImportParameterList();
			function.addComment("Import Parameters");
			Out.println("Import Parameters");
			parameterList = documentFunctionFields(destination, function,
					params);
			ArrayList<ComplexParameter> inputStructures = documentStructures(
					destination, function, params);
			function.setInputStructures(inputStructures);

			function.setInputParameters(parameterList);

			params = functionTemplate.getExportParameterList();
			function.addComment("Export Parameters");
			Out.println("Export Parameters");
			parameterList = documentFunctionFields(destination, function,
					params);
			function.setOutputParameters(parameterList);

			params = functionTemplate.getTableParameterList();
			function.addComment("Table Parameters");
			Out.println("Table Parameters");
			documentTables(destination, function, params);

		} catch (JCoException e) {
			e.printStackTrace();
		}

		return function;
	}

	/**
	 * Document the fields of a function.
	 * 
	 * @param destination
	 *            The backend destination information.
	 * @param function
	 *            The function class to receive the data.
	 * @param params
	 *            The function parameters.
	 * @return The list of parameters.
	 * @throws JCoException
	 *             JCO Exception
	 */
	private static ArrayList<SimpleParameter> documentFunctionFields(
			JCoDestination destination, Function function,
			JCoListMetaData params) throws JCoException {

		ArrayList<SimpleParameter> parameterList = new ArrayList<SimpleParameter>();

		// Invalid parameters
		if (params == null) {
			function.addComment("No parameters (null value returned).");
			Out.println("No parameters (null value returned).");

		} else {
			// No parameters
			if (params.getFieldCount() == 0) {
				function.addComment("No parameters.");
				Out.println("No parameters.");
			} else {
				// Show the parameters
				for (int i = 0; i < params.getFieldCount(); i++) {

					String comment = "  " + i + " - " + params.getName(i) + " "
							+ params.getTypeAsString(i) + "("
							+ params.getLength(i) + ") - "
							+ params.getDescription(i);
					function.addComment(comment);
					Out.println(comment);

					if (JCoMetaData.TYPE_STRUCTURE == params.getType(i)) {

						// Ignore. Do nothing at the moment.

					} else {
						SimpleParameter simpleParameter = new SimpleParameter();
						simpleParameter.setName(params.getName(i));
						simpleParameter.setLength(params.getLength(i));
						simpleParameter
								.setDescription(params.getDescription(i));
						simpleParameter.setType(params.getTypeAsString(i));
						simpleParameter.setPosition(i);

						parameterList.add(simpleParameter);

					}
				}
			}
		}

		return parameterList;
	}

	/**
	 * Document the structures of the function.
	 * 
	 * @param destination
	 *            The backend destination information.
	 * @param function
	 *            The function class to receive the data.
	 * @param params
	 *            The function parameters.
	 * @return The list of parameters.
	 * @throws JCoException
	 *             JCO Exception
	 */
	private static ArrayList<ComplexParameter> documentStructures(
			JCoDestination destination, Function function,
			JCoListMetaData params) throws JCoException {

		ArrayList<ComplexParameter> parameterList = new ArrayList<ComplexParameter>();

		// Invalid parameters
		if (params == null) {
			function.addComment("No structures (null value returned).");
			Out.println("No structures (null value returned).");

		} else {
			// No parameters
			if (params.getFieldCount() == 0) {
				function.addComment("No structures.");
				Out.println("No structures.");
			} else {
				// Show the parameters
				for (int i = 0; i < params.getFieldCount(); i++) {

					if (JCoMetaData.TYPE_STRUCTURE == params.getType(i)) {

						JCoRecordMetaData meta = params.getRecordMetaData(i);

						String comment = "Structure " + i + " - "
								+ meta.getName();
						function.addComment(comment);
						Out.println(comment);

						ComplexParameter complex = new ComplexParameter();
						complex.setName(meta.getName());
						complex.setStructureName(meta.getName());
						complex.setType(ComplexParameter.STRUCTURE_TYPE);

						ArrayList<SimpleParameter> parameters = new ArrayList<SimpleParameter>();
						for (int j = 0; j < meta.getFieldCount(); j++) {

							comment = "      " + j + " - " + meta.getName(j)
									+ " " + meta.getTypeAsString(j) + "("
									+ meta.getLength(j) + ") - "
									+ meta.getDescription(j);
							function.addComment(comment);
							Out.println(comment);

							SimpleParameter simpleParameter = new SimpleParameter();
							simpleParameter.setName(meta.getName(j));
							simpleParameter.setLength(meta.getLength(j));
							simpleParameter.setDescription(meta
									.getDescription(j));
							simpleParameter.setType(meta.getTypeAsString(j));
							simpleParameter.setPosition(j);

							parameters.add(simpleParameter);
						}
						complex.setParameters(parameters);

						parameterList.add(complex);
					}
				}
			}
		}

		return parameterList;
	}

	/**
	 * Document a table.
	 * 
	 * @param destination
	 *            The backend destination informatio.
	 * @param function
	 *            The function class to receive the data.
	 * @param params
	 *            The function parameters.
	 * @throws JCoException
	 *             JCO Exception
	 */
	private static void documentTables(JCoDestination destination,
			Function function, JCoListMetaData params) throws JCoException {

		for (int i = 0; i < params.getFieldCount(); i++) {

			Out.println("Name: " + params.getName(i));
			function.addComment("Name: " + params.getName(i));
			Out.println("Record Type Name: " + params.getRecordTypeName(i));
			function.addComment("Record Type Name: "
					+ params.getRecordTypeName(i));

			ComplexParameter complexParameter = new ComplexParameter();
			complexParameter.setName(params.getName(i));
			complexParameter.setStructureName(params.getRecordTypeName(i));
			complexParameter.setPosition(i);

			if (params.getType(i) == JCoMetaData.TYPE_TABLE) {
				complexParameter.setType(ComplexParameter.TABLE_TYPE);
			} else if (params.getType(i) == JCoMetaData.TYPE_STRUCTURE) {
				complexParameter.setType(ComplexParameter.STRUCTURE_TYPE);
			}

			JCoMetaData typeMetaData;
			typeMetaData = destination.getRepository().getStructureDefinition(
					params.getRecordTypeName(i));

			for (int j = 0; j < typeMetaData.getFieldCount(); j++) {

				String memberTypeName;

				if (typeMetaData.getType(j) == JCoMetaData.TYPE_TABLE
						|| typeMetaData.getType(j) == JCoMetaData.TYPE_STRUCTURE)
					memberTypeName = typeMetaData.getRecordMetaData(j)
							.getName();
				else
					memberTypeName = typeMetaData.getTypeAsString(j);

				String comment = "      " + j + " - " + typeMetaData.getName(j)
						+ " " + memberTypeName + "("
						+ typeMetaData.getLength(j) + ") - "
						+ typeMetaData.getDescription(j);
				function.addComment(comment);
				Out.println(comment);

				SimpleParameter simpleParameter = new SimpleParameter();
				simpleParameter.setName(typeMetaData.getName(j));
				simpleParameter.setLength(typeMetaData.getLength(j));
				simpleParameter.setDescription(typeMetaData.getDescription(j));
				simpleParameter.setType(memberTypeName);
				simpleParameter.setPosition(j);

				complexParameter.addParameters(simpleParameter);

			}

			function.addComplexParameter(complexParameter);
		}
	}
}
