/**
 * The program(s) herein may be used  freely
 * for personal use only. It may not be sold, 
 * rented, leased, sublicensed  or  modified 
 * without permission of  the  author.  
 * 
 *           www.carlosbcruz.com
 *           
 * This program is provided "AS IS"  without 
 * warranty of any kind. In no event I  will
 * be liable  for  any  direct  or  indirect 
 * damage.
 */

package com.carlosbcruz.sapproxy.generator;

import java.io.IOException;
import java.util.ArrayList;

import com.carlosbcruz.sapproxy.generator.model.ComplexParameter;
import com.carlosbcruz.sapproxy.generator.model.Function;
import com.carlosbcruz.sapproxy.generator.model.SimpleParameter;
import com.carlosbcruz.util.ExceptionSupport;
import com.carlosbcruz.util.FileSupport;
import com.carlosbcruz.util.StringSupport;

/**
 * Generate proxies
 *  
 * @author Carlos Fernando Bella Cruz - <a href="mailto:pessoal@carlosbcruz.com">pessoal@carlosbcruz.com</a>
 */
public class ProxyGenerator {

	private static final String NL = "\r\n";
	private static final String PROXY_FILE = "PortType";
	private static final String PARAMETER_FILE = "Parameters";

	/**
	 * Generate the proxy classes.
	 * 
	 * @param directory
	 *            The directory where the files will be created.
	 * @param extendsPackage
	 *            the java package name.
	 * @param documentedFunction
	 *            The function structure.
	 */
	public static void generateProxy(String directory, String extendsPackage,
			Function function) {

		// Create the class
		String targetDirectory = directory
				+ "/"
				+ StringSupport.replaceCharacters(function.getPackageName(),
						'.', '/');
		FileSupport.createDirectory(targetDirectory);

		ArrayList<ComplexParameter> structures = function.getInputStructures();
		for (ComplexParameter structure : structures) {

			StringBuffer tableClass = generateParameterClass(targetDirectory,
					function.getPackageName(), extendsPackage, structure
							.getName(), structure.getParameters(),
					new ArrayList<ComplexParameter>());
			String fileName = getUpperName(structure.getName()) + ".java";
			FileSupport.writeTextFile(targetDirectory, fileName, tableClass
					.toString());
			formatCode(targetDirectory, fileName);
		}

		String functionName = getUpperName(function.getName() + "_Input");
		StringBuffer inputClass = generateParameterClass(targetDirectory,
				function.getPackageName(), extendsPackage, functionName,
				function.getInputParameters(), function.getInputStructures());
		String fileName = functionName + ".java";
		FileSupport.writeTextFile(targetDirectory, fileName, inputClass
				.toString());
		formatCode(targetDirectory, fileName);

		functionName = getUpperName(function.getName() + "_Output");
		StringBuffer outputClass = generateParameterClass(targetDirectory,
				function.getPackageName(), extendsPackage, functionName,
				function.getOutputParameters(),
				new ArrayList<ComplexParameter>());
		fileName = functionName + ".java";
		FileSupport.writeTextFile(targetDirectory, fileName, outputClass
				.toString());
		formatCode(targetDirectory, fileName);

		for (int i = 0; i < function.getNumberOfComplexParameters(); i++) {

			ComplexParameter parameter = function.getComplexParameter(i);

			StringBuffer tableClass = generateParameterClass(targetDirectory,
					function.getPackageName(), extendsPackage, parameter
							.getName(), parameter.getParameters(),
					new ArrayList<ComplexParameter>());
			fileName = getUpperName(parameter.getName()) + ".java";
			FileSupport.writeTextFile(targetDirectory, fileName, tableClass
					.toString());
			formatCode(targetDirectory, fileName);

		}

		StringBuffer portTypeClass = generatePortTypeClass(targetDirectory,
				function.getPackageName(), extendsPackage, function);
		fileName = getUpperName(function.getName()) + "_PortType.java";
		FileSupport.writeTextFile(targetDirectory, fileName, portTypeClass
				.toString());
		formatCode(targetDirectory, fileName);
	}

	/**
	 * Generate the main port type.
	 * 
	 * @param directory
	 *            the directory to generate the file.
	 * @param extendsPackage
	 *            The main port type package.
	 */
	public static void generateMainPortType(String directory,
			String extendsPackage) {

		StringBuffer out = new StringBuffer();

		out.append("package " + extendsPackage + ";" + NL);

		out.append("import com.sap.conn.jco.JCoDestination;" + NL);
		out.append("import com.sap.conn.jco.JCoException;" + NL);

		out.append("public abstract class " + PROXY_FILE + " {" + NL);

		out.append("protected JCoDestination destination = null;" + NL);

		out.append("public " + PROXY_FILE + "(JCoDestination destination) {"
				+ NL);
		out.append("this.destination = destination;" + NL);
		out.append("}" + NL);
		out.append(NL);

		out
				.append("protected abstract Parameters execute(Parameters parameters) throws JCoException;"
						+ NL);

		out.append(NL);

		out.append("}" + NL);

		// Crete the class
		String targetDirectory = directory + "/"
				+ StringSupport.replaceCharacters(extendsPackage, '.', '/');
		FileSupport.createDirectory(targetDirectory);

		FileSupport.writeTextFile(targetDirectory, PROXY_FILE + ".java", out
				.toString());
		formatCode(targetDirectory, PROXY_FILE + ".java");

		out = new StringBuffer();

		out.append("package " + extendsPackage + ";" + NL);

		out.append("public class " + PARAMETER_FILE + " {" + NL);

		out.append("}" + NL);

		// Crete the Parameters class
		FileSupport.writeTextFile(targetDirectory, PARAMETER_FILE + ".java",
				out.toString());
		formatCode(targetDirectory, PARAMETER_FILE + ".java");

	}

	/**
	 * Generate the class for the input parameters.
	 * 
	 * @param directory
	 *            The directory to generate the file.
	 * @param packageName
	 *            The package name for the class.
	 * @param extendsPackage
	 *            The class to extend from.
	 * @param functionName
	 *            The function name being generated.
	 * @param parameterList
	 *            The parameter list.
	 * @return
	 */
	private static StringBuffer generateParameterClass(String directory,
			String packageName, String extendsPackage, String functionName,
			ArrayList<SimpleParameter> parameterList,
			ArrayList<ComplexParameter> structureList) {

		StringBuffer out = new StringBuffer();

		out.append("package " + packageName + ";" + NL);
		out.append(NL);

		boolean hasDate = false;
		for (SimpleParameter parameter : parameterList) {
			if ("DATE".equals(parameter.getType())) {
				hasDate = true;
			}
		}
		if (hasDate) {
			out.append("import java.util.Date;" + NL);
			out.append(NL);
		}

		out.append("public class " + getUpperName(functionName) + " extends "
				+ extendsPackage + "." + PARAMETER_FILE + " {" + NL);
		out.append(NL);

		for (SimpleParameter parameter : parameterList) {

			if ("CHAR".equals(parameter.getType())) {
				out.append("private String "
						+ getLowerName(parameter.getName())
						+ " = new String();  // " + parameter.getDescription()
						+ NL);
			}

			if ("NUM".equals(parameter.getType())
					|| "BCD".equals(parameter.getType())) {
				out.append("private int " + getLowerName(parameter.getName())
						+ " = 0;  // " + parameter.getDescription() + NL);
			}

			if ("DATE".equals(parameter.getType())) {
				out.append("private Date " + getLowerName(parameter.getName())
						+ " = new Date();  // " + parameter.getDescription()
						+ NL);
			}
		}

		out.append(NL);

		for (ComplexParameter structure : structureList) {

			out.append("private " + getUpperName(structure.getName()) + " "
					+ getLowerName(structure.getName()) + " = new "
					+ getUpperName(structure.getName()) + "();  // Structure "
					+ NL);
		}

		for (SimpleParameter parameter : parameterList) {

			out.append("/**" + NL);
			out.append(" * Set: " + parameter.getDescription() + NL);
			out.append(" * " + NL);
			out.append(" * @param " + getLowerName(parameter.getName()) + NL);
			out.append(" */" + NL);

			// Generate the set method.
			if ("CHAR".equals(parameter.getType())) {
				out.append("public void set"
						+ getUpperName(parameter.getName()) + "(String "
						+ getLowerName(parameter.getName()) + ") {" + NL);

				out.append("this." + getLowerName(parameter.getName()) + "="
						+ getLowerName(parameter.getName()) + ";" + NL);

				out.append("}" + NL);
			}

			if ("NUM".equals(parameter.getType())
					|| "BCD".equals(parameter.getType())) {
				out.append("public void set"
						+ getUpperName(parameter.getName()) + "(int "
						+ getLowerName(parameter.getName()) + ") {" + NL);

				out.append("this." + getLowerName(parameter.getName()) + "="
						+ getLowerName(parameter.getName()) + ";" + NL);

				out.append("}" + NL);
			}

			if ("DATE".equals(parameter.getType())) {
				out.append("public void set"
						+ getUpperName(parameter.getName()) + "(Date "
						+ getLowerName(parameter.getName()) + ") {" + NL);

				out.append("this." + getLowerName(parameter.getName()) + "="
						+ getLowerName(parameter.getName()) + ";" + NL);

				out.append("}" + NL);
			}

			out.append(NL);

			out.append("/**" + NL);
			out.append(" * Get: " + parameter.getDescription() + NL);
			out.append(" * " + NL);
			out.append(" * return " + getLowerName(parameter.getName()) + NL);
			out.append(" */" + NL);

			// Generate the get method.
			if ("CHAR".equals(parameter.getType())) {
				out.append("public String get"
						+ getUpperName(parameter.getName()) + "() {" + NL);
				out.append("return " + getLowerName(parameter.getName()) + ";"
						+ NL);
				out.append("}" + NL);
			}
			if ("NUM".equals(parameter.getType())
					|| "BCD".equals(parameter.getType())) {
				out.append("public int get" + getUpperName(parameter.getName())
						+ "() {" + NL);
				out.append("return " + getLowerName(parameter.getName()) + ";"
						+ NL);
				out.append("}" + NL);
			}
			if ("DATE".equals(parameter.getType())) {
				out.append("public Date get"
						+ getUpperName(parameter.getName()) + "() {" + NL);
				out.append("return " + getLowerName(parameter.getName()) + ";"
						+ NL);
				out.append("}" + NL);
			}

			out.append(NL);
		}

		out.append(NL);

		for (ComplexParameter structure : structureList) {

			out.append("/**" + NL);
			out.append(" * Provide the " + getUpperName(structure.getName())
					+ " structure." + NL);
			out.append(" * @return The " + getUpperName(structure.getName())
					+ " structure." + NL);
			out.append(" */" + NL);
			out.append("public " + getUpperName(structure.getName()) + " get"
					+ getUpperName(structure.getName()) + "() {" + NL);
			out.append("	return " + getLowerName(structure.getName()) + ";"
					+ NL);
			out.append("}" + NL);

			out.append(NL);

			out.append("/** Set the " + getUpperName(structure.getName())
					+ " structure." + NL);
			out.append(" * @param " + getLowerName(structure.getName())
					+ " The " + getUpperName(structure.getName())
					+ " structure" + NL);
			out.append(" */" + NL);
			out.append("public void set" + getUpperName(structure.getName())
					+ "(" + getUpperName(structure.getName()) + " "
					+ getLowerName(structure.getName()) + ") {" + NL);
			out.append("	this." + getLowerName(structure.getName()) + " = "
					+ getLowerName(structure.getName()) + ";" + NL);
			out.append("}" + NL);
		}
		out.append(NL);

		out.append("@Override" + NL);
		out.append("public String toString() {" + NL);

		out.append("String out = new String();" + NL);

		out.append("out += \"" + getUpperName(functionName) + " [\";" + NL);

		for (SimpleParameter parameter : parameterList) {

			out.append("out += \" " + getLowerName(parameter.getName())
					+ ":[\" + " + getLowerName(parameter.getName())
					+ " + \"]\";" + NL);
		}
		out.append("out += \"]\";" + NL);

		out.append("return out;" + NL);
		out.append("}" + NL);

		out.append("}" + NL);

		return out;
	}

	/**
	 * Generate the port type class.
	 * 
	 * @param directory
	 *            The directory.
	 * @param packageName
	 *            The package name.
	 * @param extendsPackage
	 *            The package name.
	 * @param function
	 *            The function.
	 * @return
	 */
	private static StringBuffer generatePortTypeClass(String directory,
			String packageName, String extendsPackage, Function function) {

		StringBuffer out = new StringBuffer();

		out.append(getJavaComment(function.getComment()));
		out.append("package " + packageName + ";" + NL);
		out.append(NL);

		out.append("import java.util.ArrayList;" + NL);
		out
				.append("import " + extendsPackage + "." + PARAMETER_FILE + ";"
						+ NL);
		out.append("import com.sap.conn.jco.JCoDestination;" + NL);
		out.append("import com.sap.conn.jco.JCoException;" + NL);
		out.append("import com.sap.conn.jco.JCoFunction;" + NL);
		out.append("import com.sap.conn.jco.JCoParameterList;" + NL);

		ArrayList<ComplexParameter> inputStructures = function
				.getInputStructures();
		if (inputStructures.size() > 0) {
			out.append("import com.sap.conn.jco.JCoStructure;" + NL);
		}
		out.append("import com.sap.conn.jco.JCoTable;" + NL);

		out.append(NL);

		out.append("/**" + NL);
		out.append(" * Access class for function "
				+ getUpperName(function.getName()) + NL);
		out.append(" *" + NL);
		out.append(" * @author SAP Proxy Generator." + NL);
		out.append(" */" + NL);
		out.append("public class " + getUpperName(function.getName())
				+ "_PortType " + " extends " + extendsPackage + ".PortType  {"
				+ NL);

		out.append(NL);

		String inputClassName = getUpperName(function.getName()) + "_Input";
		String inputFieldName = getLowerName(function.getName()) + "_Input";
		String outputClassName = getUpperName(function.getName()) + "_Output";
		String outputFieldName = getLowerName(function.getName()) + "_Output";

		out.append("private " + inputClassName + " " + inputFieldName
				+ " = null;" + NL);

		out.append("private " + outputClassName + " " + outputFieldName
				+ " = null;" + NL);
		out.append("private JCoParameterList inputParameterList = null;" + NL);
		out.append("private JCoParameterList outputParameterList = null;" + NL);
		out.append("private JCoParameterList tables =  null;" + NL);
		out.append("private JCoFunction function = null;" + NL);

		for (int i = 0; i < function.getNumberOfComplexParameters(); i++) {

			ComplexParameter parameter = function.getComplexParameter(i);

			out.append("private ArrayList<" + getUpperName(parameter.getName())
					+ "> " + getLowerName(parameter.getName())
					+ "_List = new ArrayList<"
					+ getUpperName(parameter.getName()) + ">();" + NL);

		}

		out.append(NL);

		out.append("/**" + NL);
		out.append(" * Default constructor." + NL);
		out.append(" *" + NL);
		out.append(" * @param destination The SAP destination information."
				+ NL);
		out.append(" */" + NL);
		out.append("public " + getUpperName(function.getName())
				+ "_PortType (JCoDestination destination) {" + NL);
		out.append("super(destination);" + NL);
		out.append("}" + NL);

		for (int i = 0; i < function.getNumberOfComplexParameters(); i++) {

			ComplexParameter parameter = function.getComplexParameter(i);

			String className = getUpperName(parameter.getName());
			String fieldName = getLowerName(parameter.getName());

			out.append("/**" + NL);
			out.append(" * Get the number of "
					+ getUpperName(parameter.getName()) + " items." + NL);
			out.append(" * @return The number of "
					+ getUpperName(parameter.getName()) + " item." + NL);
			out.append(" */" + NL);
			out.append("public int get" + className + "Size() {" + NL);
			out.append("return " + fieldName + "_List.size();" + NL);
			out.append("}" + NL);

			out.append("/**" + NL);
			out.append(" * Get a specific " + getUpperName(parameter.getName())
					+ "." + NL);
			out.append(" * @param i The position requested." + NL);
			out.append(" * @return The " + getUpperName(parameter.getName())
					+ " at the position requested." + NL);
			out.append(" */" + NL);
			out.append("public " + className + " get" + className
					+ "AtRow(int i) {" + NL);
			out.append("return " + fieldName + "_List.get(i);" + NL);
			out.append("}" + NL);

			out.append("/**" + NL);
			out.append(" * Add a " + getUpperName(parameter.getName())
					+ " item." + NL);
			out.append(" * @param " + fieldName + " A "
					+ getUpperName(parameter.getName()) + " to be added." + NL);
			out.append(" */" + NL);
			out.append("public void add" + className + "(" + className + " "
					+ fieldName + ") {" + NL);
			out.append(fieldName + "_List.add(" + fieldName + ");" + NL);
			out.append("}" + NL);

		}

		out.append("/*" + NL);
		out.append(" * (non-Javadoc)" + NL);
		out.append(" *" + NL);
		out.append(" * @see " + extendsPackage + ".PortType#execute("
				+ extendsPackage + ".Parameters)" + NL);
		out.append(" */" + NL);

		out
				.append("public Parameters execute(Parameters parameters) throws JCoException"
						+ NL);
		out.append("{" + NL);
		out.append("function = destination.getRepository().getFunction(\""
				+ function.getName() + "\");" + NL);

		out.append("if(function == null) {" + NL);
		out.append("throw new RuntimeException(\"" + function.getName()
				+ " not found in SAP.\");" + NL);
		out.append("}" + NL);

		out.append(NL);
		out.append(inputFieldName + " = (" + inputClassName + ") parameters;"
				+ NL);
		out.append(NL);

		out.append("inputParameterList = function.getImportParameterList();"
				+ NL);

		out.append(NL);

		// Generate input with structure code.
		if (inputStructures.size() > 0) {

			for (ComplexParameter structure : inputStructures) {

				out
						.append("JCoStructure "
								+ getLowerName(structure.getName())
								+ "Structure = function.getImportParameterList().getStructure(\""
								+ structure.getName() + "\");" + NL);

				out.append(getUpperName(structure.getName()) + " "
						+ getLowerName(structure.getName()) + " = "
						+ inputFieldName + ".get"
						+ getUpperName(structure.getName()) + "();" + NL);

				ArrayList<SimpleParameter> parameters = structure
						.getParameters();
				for (SimpleParameter parameter : parameters) {
					out.append(getLowerName(structure.getName())
							+ "Structure.setValue(" + parameter.getPosition()
							+ ", " + getLowerName(structure.getName()) + ".get"
							+ getUpperName(parameter.getName()) + "());" + NL);
				}
			}
		}
		out.append(NL);

		// Generate the settings of the input parameters.
		ArrayList<SimpleParameter> inputFields = function.getInputParameters();
		for (SimpleParameter parameter : inputFields) {
			out.append("inputParameterList.setValue(" + parameter.getPosition()
					+ ", " + inputFieldName + ".get"
					+ getUpperName(parameter.getName()) + "()); // "
					+ parameter.getDescription() + NL);
		}
		out.append(NL);

		out.append("tables = function.getTableParameterList();" + NL);
		out.append(NL);

		for (int i = 0; i < function.getNumberOfComplexParameters(); i++) {
			ComplexParameter complex = function.getComplexParameter(i);

			out.append("if (!" + getLowerName(complex.getName())
					+ "_List.isEmpty()) {" + NL);

			out.append(NL);
			out.append("JCoTable " + getLowerName(complex.getName())
					+ "_Table = tables.getTable(" + complex.getPosition()
					+ ");" + NL);
			out.append(NL);

			out.append("for (" + getUpperName(complex.getName()) + " "
					+ getLowerName(complex.getName()) + "_Item: "
					+ getLowerName(complex.getName()) + "_List) {" + NL);

			out.append(getLowerName(complex.getName()) + "_Table.appendRow();"
					+ NL);

			ArrayList<SimpleParameter> parameters = complex.getParameters();
			for (SimpleParameter parameter : parameters) {

				out.append(getLowerName(complex.getName()) + "_Table.setValue("
						+ parameter.getPosition() + ", "
						+ getLowerName(complex.getName()) + "_Item.get"
						+ getUpperName(parameter.getName()) + "());" + NL);
			}

			out.append("}" + NL);
			out.append(NL);

			out.append("}");
			out.append(NL);

		}
		out.append(NL);

		out.append("function.execute(destination);" + NL);

		out.append(NL);

		// Generate the settings of the output parameters.
		ArrayList<SimpleParameter> outputFields = function
				.getOutputParameters();

		if (outputFields.size() > 0) {
			out
					.append("outputParameterList = function.getExportParameterList();"
							+ NL);

			out.append(NL);

			out.append(outputFieldName + " = new " + outputClassName + "();"
					+ NL);

			for (int i = 0; i < outputFields.size(); i++) {

				SimpleParameter parameter = outputFields.get(i);

				String type = new String();
				if ("CHAR".equals(parameter.getType())) {
					type = "String";
				}
				if ("NUM".equals(parameter.getType())
						|| "BCD".equals(parameter.getType())) {
					type = "Int";
				}
				if ("DATE".equals(parameter.getType())) {
					type = "Date";
				}

				out.append(outputFieldName + ".set"
						+ getUpperName(parameter.getName())
						+ "(outputParameterList.get" + type + "(" + i
						+ ")); // " + parameter.getDescription() + NL);
			}
			out.append(NL);
		}

		for (int i = 0; i < function.getNumberOfComplexParameters(); i++) {
			ComplexParameter complex = function.getComplexParameter(i);

			out.append("JCoTable " + getLowerName(complex.getName())
					+ "_Table = tables.getTable(" + complex.getPosition()
					+ ");" + NL);

			out.append("for (int i = 0; i < " + getLowerName(complex.getName())
					+ "_Table.getNumRows(); i++) {" + NL);

			out.append(getLowerName(complex.getName()) + "_Table.setRow(i);"
					+ NL);

			out.append(getUpperName(complex.getName()) + " row = new "
					+ getUpperName(complex.getName()) + "();" + NL);

			ArrayList<SimpleParameter> parameters = complex.getParameters();
			for (int j = 0; j < parameters.size(); j++) {

				SimpleParameter parameter = parameters.get(j);

				String type = new String();
				if ("CHAR".equals(parameter.getType())) {
					type = "String";
				}
				if ("NUM".equals(parameter.getType())
						|| "BCD".equals(parameter.getType())) {
					type = "Int";
				}
				if ("DATE".equals(parameter.getType())) {
					type = "Date";
				}

				out.append("row.set" + getUpperName(parameter.getName()) + "("
						+ getLowerName(complex.getName()) + "_Table.get" + type
						+ "(" + parameter.getPosition() + ")); // "
						+ parameter.getDescription() + NL);

			}

			out
					.append(getLowerName(complex.getName()) + "_List.add(row);"
							+ NL);

			out.append("}");
		}

		out.append(NL);
		out.append("return " + outputFieldName + ";" + NL);
		out.append(NL);

		out.append("}" + NL);

		for (int i = 0; i < function.getNumberOfComplexParameters(); i++) {
			ComplexParameter complex = function.getComplexParameter(i);

			out.append("/**" + NL);
			out.append(" * Provide the list of "
					+ getUpperName(complex.getName()) + " items." + NL);
			out.append(" * @return The list of "
					+ getUpperName(complex.getName()) + " items." + NL);
			out.append(" */" + NL);
			out.append("public ArrayList<" + getUpperName(complex.getName())
					+ "> get" + getUpperName(complex.getName()) + "_List() {"
					+ NL);
			out.append("return " + getLowerName(complex.getName()) + "_List;"
					+ NL);
			out.append("}" + NL);

			out.append("/**" + NL);
			out.append(" * Set the list of " + getUpperName(complex.getName())
					+ " items." + NL);
			out.append(" * @param " + getLowerName(complex.getName())
					+ "_List The list of " + getUpperName(complex.getName())
					+ " items." + NL);
			out.append(" */" + NL);
			out.append("public void set" + getUpperName(complex.getName())
					+ "_List(ArrayList<" + getUpperName(complex.getName())
					+ "> " + getLowerName(complex.getName()) + "_List) {" + NL);
			out.append("this." + getLowerName(complex.getName()) + "_List = "
					+ getLowerName(complex.getName()) + "_List;" + NL);
			out.append("}" + NL);

			out.append(NL);
		}

		for (int i = 0; i < function.getNumberOfComplexParameters(); i++) {
			ComplexParameter complex = function.getComplexParameter(i);

			out.append("/**" + NL);
			out.append(" * Add a " + getUpperName(complex.getName())
					+ " element to the list." + NL);
			out.append(" * @param " + getLowerName(complex.getName())
					+ "_Item The " + getUpperName(complex.getName())
					+ " element." + NL);
			out.append(" */" + NL);
			out.append("public void add" + getUpperName(complex.getName())
					+ "Item(" + getUpperName(complex.getName()) + " "
					+ getLowerName(complex.getName()) + "_Item)" + NL);
			out.append("{" + NL);
			out.append(getLowerName(complex.getName()) + "_List.add("
					+ getLowerName(complex.getName()) + "_Item);" + NL);
			out.append("}" + NL);

			out.append(NL);
		}

		out.append("@Override" + NL);
		out.append("public String toString() {" + NL);

		out.append("String out = new String();" + NL);

		out.append("out += \"" + getUpperName(function.getName())
				+ "_PortType\\n\";" + NL);

		out.append("out += \"Import Parameters\\n\";" + NL);
		for (SimpleParameter parameter : function.getInputParameters()) {

			out
					.append("out += \"     "
							+ getLowerName(parameter.getName()) + ":[\" + "
							+ inputFieldName + ".get"
							+ getUpperName(parameter.getName())
							+ "() + \"]\\n\";" + NL);
		}

		out.append("out += \"Export Parameters\\n\";" + NL);
		for (SimpleParameter parameter : function.getOutputParameters()) {

			out
					.append("out += \"     "
							+ getLowerName(parameter.getName()) + ":[\" + "
							+ outputFieldName + ".get"
							+ getUpperName(parameter.getName())
							+ "() + \"]\\n\";" + NL);
		}
		out.append("out += \"Tables\\n\";" + NL);

		for (int i = 0; i < function.getNumberOfComplexParameters(); i++) {
			ComplexParameter complex = function.getComplexParameter(i);

			out.append("" + NL);

			out.append("for (" + getUpperName(complex.getName()) + " "
					+ getLowerName(complex.getName()) + "_Item: "
					+ getLowerName(complex.getName()) + "_List) {" + NL);
			out.append("out += " + getLowerName(complex.getName())
					+ "_Item.toString() + \"\\n\";" + NL);
			out.append("}" + NL);

			out.append(NL);

		}

		out.append("return out;" + NL);
		out.append("}" + NL);

		out.append(NL);

		out.append("/**" + NL);
		out
				.append(" * Convert the inputParameterList into a XML content."
						+ NL);
		out.append(" * @return The content in XML format." + NL);
		out.append(" */" + NL);
		out.append("public String getInputXML(){" + NL);
		out.append("return inputParameterList.toXML();" + NL);
		out.append("}" + NL);

		out.append(NL);

		out.append("/**" + NL);
		out.append(" * Convert the outputParameterList into a XML content."
				+ NL);
		out.append(" * @return The content in XML format." + NL);
		out.append(" */" + NL);
		out.append("public String getOutputXML(){" + NL);
		out.append("return outputParameterList.toXML();" + NL);
		out.append("}" + NL);

		out.append(NL);

		out.append("/**" + NL);
		out.append(" * Convert the tables into a XML content." + NL);
		out.append(" * @return The content in XML format." + NL);
		out.append(" */" + NL);
		out.append("public String getTablesXML(){" + NL);
		out.append("return tables.toXML();" + NL);
		out.append("}" + NL);

		out.append(NL);

		out.append("/**" + NL);
		out.append(" * Convert the function into a XML content." + NL);
		out.append(" * @return The content in XML format." + NL);
		out.append(" */" + NL);
		out.append("public String getFunctionXML(){" + NL);

		out.append("return function.toXML();" + NL);
		out.append("}" + NL);

		out.append("}" + NL);

		return out;
	}

	/**
	 * Generate a java comment from a specific text.
	 * 
	 * @param text
	 *            The text to be inserted on a comment.
	 * @return The test on a java comment format.
	 */
	private static String getJavaComment(String text) {

		StringBuffer out = new StringBuffer();

		out.append("/**" + NL);

		String[] lines = StringSupport.breakContentBasedOnDelimiter(text, NL);

		for (int i = 0; i < lines.length; i++) {

			out.append(" * " + lines[i] + NL);
		}
		out.append(" */" + NL);

		return out.toString();
	}

	/**
	 * Transform a name into a upper name.
	 * 
	 * @param name
	 *            The name to be transformed.
	 * @return The name in upper name format.
	 */
	private static String getUpperName(String name) {

		String className = new String();

		className += Character.toUpperCase(name.charAt(0));

		boolean nextIsUpper = false;
		for (int i = 1; i < name.length(); i++) {

			if (nextIsUpper) {
				className += Character.toUpperCase(name.charAt(i));
			} else {
				className += Character.toLowerCase(name.charAt(i));
			}

			nextIsUpper = false;
			if ('_' == name.charAt(i)) {
				nextIsUpper = true;
			}
		}

		return className;
	}

	/**
	 * Transform a name into a lower name.
	 * 
	 * @param name
	 *            The name to be transformed.
	 * @return The name in lower name format.
	 */
	private static String getLowerName(String name) {

		String className = new String();

		className += Character.toLowerCase(name.charAt(0));

		boolean nextIsUpper = false;
		for (int i = 1; i < name.length(); i++) {

			if (nextIsUpper) {
				className += Character.toUpperCase(name.charAt(i));
			} else {
				className += Character.toLowerCase(name.charAt(i));
			}

			nextIsUpper = false;
			if ('_' == name.charAt(i)) {
				nextIsUpper = true;
			}
		}

		return className;
	}

	/**
	 * Format a specific code.
	 * 
	 * @param directory
	 *            The directory of the code.
	 * @param fileName
	 *            The file name.
	 */
	public static void formatCode(String directory, String fileName) {

		String command = "uncrustify --replace --no-backup -c ben.cfg "
				+ FileSupport.getFileName(directory, fileName);
		try {
			Runtime.getRuntime().exec(command);
		} catch (IOException e) {
			ExceptionSupport.handleException(e);
		}

	}
}
